@@ -54,7 +54,6 @@ struct ldap_server_s
   unsigned int starttls:1;       /* Use STARTTLS.  */
   unsigned int ldap_over_tls:1;  /* Use LDAP over an TLS tunnel */
   unsigned int ntds:1;           /* Use Active Directory authentication.  */
-  unsigned int areconly:1;       /* Set LDAP_OPT_AREC_EXCLUSIVE.  */
 };
 typedef struct ldap_server_s *ldap_server_t;
 
@@ -188,10 +187,8 @@ struct cert_ref_s
 };
 typedef struct cert_ref_s *cert_ref_t;
 
-/* Forward reference; access only via ks-engine-ldap.c.  */
-struct ks_engine_ldap_local_s;
 
-/* Forward reference; access only through server.c.  */
+/* Forward references; access only through server.c.  */
 struct server_local_s;
 
 #if SIZEOF_UNSIGNED_LONG == 8
@@ -208,7 +205,6 @@ struct server_control_s
   int no_server;      /* We are not running under server control. */
   int status_fd;      /* Only for non-server mode. */
   struct server_local_s *server_local;
-  struct ks_engine_ldap_local_s *ks_get_state;
   int force_crl_refresh; /* Always load a fresh CRL. */
 
   int check_revocations_nest_level; /* Internal to check_revovations.  */
