@@ -21,7 +21,6 @@
 
 #include <config.h>
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -49,6 +48,7 @@
 
 #include <gpg-error.h>
 #include "../common/logging.h"
+#include "../common/argparse.h"
 #include "../common/stringhelp.h"
 #include "../common/mischelp.h"
 #include "../common/strlist.h"
@@ -92,14 +92,14 @@ enum
     oStartTLS,
     oLdapTLS,
     oNtds,
-    oARecOnly,
+
     oOnlySearchTimeout,
     oLogWithPID
   };
 
 
 /* The list of options as used by the argparse.c code.  */
-static gpgrt_opt_t opts[] = {
+static ARGPARSE_OPTS opts[] = {
   { oVerbose,  "verbose",   0, "verbose" },
   { oQuiet,    "quiet",     0, "be somewhat more quiet" },
   { oTimeout,  "timeout",   1, "|N|set LDAP timeout to N seconds"},
@@ -107,10 +107,9 @@ static gpgrt_opt_t opts[] = {
                                " a record oriented format"},
   { oProxy,    "proxy",     2,
                 "|NAME|ignore host part and connect through NAME"},
-  { oStartTLS, "starttls",  0, "use STARTLS for the connection"},
+  { oStartTLS, "starttls",  0, "use STARTLS for the conenction"},
   { oLdapTLS,  "ldaptls",   0, "use a TLS for the connection"},
   { oNtds,     "ntds",      0, "authenticate using AD"},
-  { oARecOnly, "areconly",  0, "do only an A record lookup"},
   { oHost,     "host",      2, "|NAME|connect to host NAME"},
   { oPort,     "port",      1, "|N|connect to port N"},
   { oUser,     "user",      2, "|NAME|use NAME for authentication"},
@@ -136,7 +135,6 @@ static struct
   int starttls;
   int ldaptls;
   int ntds;
-  int areconly;
 
   estream_t outstream;    /* Send output to this stream.  */
 
@@ -167,7 +165,7 @@ my_strusage (int level)
 {
   const char *p;
 
-  switch (level)
+  switch(level)
     {
     case  9: p = "GPL-3.0-or-later"; break;
     case 11: p = "dirmngr_ldap (@GNUPG@)";
@@ -196,7 +194,7 @@ my_strusage (int level)
 int
 main (int argc, char **argv)
 {
-  gpgrt_argparse_t pargs;
+  ARGPARSE_ARGS pargs;
   int any_err = 0;
   char *p;
   int only_search_timeout = 0;
@@ -205,7 +203,7 @@ main (int argc, char **argv)
 
   early_system_init ();
 
-  gpgrt_set_strusage (my_strusage);
+  set_strusage (my_strusage);
   log_set_prefix ("dirmngr_ldap", GPGRT_LOG_WITH_PREFIX);
 
   init_common_subsystems (&argc, &argv);
@@ -222,7 +220,7 @@ main (int argc, char **argv)
   pargs.argc = &argc;
   pargs.argv = &argv;
   pargs.flags= ARGPARSE_FLAG_KEEP;
-  while (gpgrt_argparse (NULL, &pargs, opts))
+  while (gnupg_argparse (NULL, &pargs, opts))
     {
       switch (pargs.r_opt)
         {
@@ -237,7 +235,6 @@ main (int argc, char **argv)
         case oStartTLS: opt.starttls = 1; opt.ldaptls = 0; break;
         case oLdapTLS:  opt.starttls = 0; opt.ldaptls = 1; break;
         case oNtds:     opt.ntds = 1; break;
-        case oARecOnly: opt.areconly = 1; break;
         case oMulti: opt.multi = 1; break;
         case oUser: opt.user = pargs.r.ret_str; break;
         case oPass: opt.pass = pargs.r.ret_str; break;
@@ -262,7 +259,7 @@ main (int argc, char **argv)
           break;
 	}
     }
-  gpgrt_argparse (NULL, &pargs, NULL);
+  gnupg_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
 
   if (only_search_timeout)
     opt.alarm_timeout = 0;
@@ -385,7 +382,7 @@ set_timeout (void)
           sec_attr.nLength = sizeof sec_attr;
           sec_attr.bInheritHandle = FALSE;
 
-          /* Create a manual resettable timer.  */
+          /* Create a manual resetable timer.  */
           timer = CreateWaitableTimer (NULL, TRUE, NULL);
           /* Initially set the timer.  */
           SetWaitableTimer (timer, &due_time, 0, NULL, NULL, 0);
@@ -440,17 +437,6 @@ connect_ldap (LDAP **r_ld)
                  opt.host, opt.port, ldap_err2string (lerr));
       goto leave;
     }
-  if (opt.areconly)
-    {
-      lerr = ldap_set_option (ld, LDAP_OPT_AREC_EXCLUSIVE, LDAP_OPT_ON);
-      if (lerr != LDAP_SUCCESS)
-        {
-          log_error ("ldap: unable to set AREC_EXLUSIVE: %s\n",
-                     ldap_err2string (lerr));
-          err = ldap_err_to_gpg_err (lerr);
-          goto leave;
-        }
-    }
 #else /* Unix */
   tmpstr = xtryasprintf ("%s://%s:%d",
                          opt.ldaptls? "ldaps" : "ldap",
@@ -761,7 +747,7 @@ print_ldap_entries (LDAP *ld, LDAPMessage *msg, char *want_attr)
 
 /* Fetch data from the server at LD using FILTER.  */
 static int
-fetch_ldap (LDAP *ld, const char *base, int scope, const char *filter)
+fetch_ldap (LDAP *ld, char *base, int scope, char *filter)
 {
   gpg_error_t err;
   int lerr;
@@ -823,7 +809,7 @@ fetch_ldap (LDAP *ld, const char *base, int scope, const char *filter)
 /* Main processing.  Take the filter and run the LDAP query. The
  * result is printed to stdout, errors are logged to the log stream.
  * To allow searching with a different base it is possible to extend
- * the filter.  For example:
+ * the filer.  For example:
  *
  *   ^CN=foo, OU=My Users&(objectClasses=*)
  *
